import { Theme } from '@emotion/react';
import { Dimensions, StyleProp, TextStyle, ViewStyle } from 'react-native';

type StyleProps = StyleProp<ViewStyle | TextStyle>;

// Function to apply style based on dimension constraints
const applyStyleIfWithinRange = (value: number, nextValue: number, style: StyleProps) => {
	const windowWidth = Dimensions.get('window').width;
	if (value <= windowWidth && (nextValue >= windowWidth || value === nextValue)) {
		return style;
	}
	return null;
};

export const mQ = (theme: Theme) => {
	const brs = theme.breakpoints;

	// Use Object.entries and reduce to create the 'breakpoints' object
	return Object.entries(brs).reduce(
		(acc, [key, value], i, entries) => {
			// Get next breakpoint value, and default to the last breakpoint value in case of undefined
			const nextValue = brs[entries[i + 1]?.[0]] ?? Object.values(brs).at(-1);

			// Assign computed function to the respective breakpoint key in the accumulator object
			acc[key] = (style: StyleProps) => applyStyleIfWithinRange(value, nextValue, style);

			// Return
			return acc;
		},
		{} as Record<keyof Theme['breakpoints'], (style: StyleProp<ViewStyle>) => object>,
	);
};
